// $Id$
//
// Copyright (C) 2010

/** \file
    \brief InternalNetworkAccessManager non-inline non-template implementation */

#include "Publisher.hh"
//#include "Publisher.ih"

// Custom includes
#include <iostream>
#include <QNetworkProxy>
#include <QVariant>
#include "Publisher.hh"
#include <boost/python.hpp>

//#include "Publisher.mpp"
#define prefix_
///////////////////////////////cc.p////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// pykit::detail::InternalServerReply

namespace pykit {
namespace detail {

    class InternalServerReply
        : public QNetworkReply
    {
        Q_OBJECT;
    public:
        InternalServerReply(QNetworkAccessManager::Operation operation,
                            QNetworkRequest const & networkRequest,
                            Publisher * publisher);

        // Forwarded to buffer
        virtual bool atEnd() const;
        virtual qint64 bytesAvailable() const;
        virtual qint64 bytesToWrite() const;
        virtual bool canReadLine() const;
        virtual bool isSequential() const;
        virtual qint64 pos() const;
        virtual bool reset();
        virtual bool seek(qint64 pos);
        virtual qint64 size() const;
        virtual bool waitForBytesWritten(int msecs);
        virtual bool waitForReadyRead(int msecs);

        virtual void abort();

        using QNetworkReply::setHeader;

        void clearResponse();

    protected:
        virtual qint64 readData(char * data, qint64 maxSize);
        virtual qint64 writeData(char const * data, qint64 maxSize);

    signals:
        void initSignal();

    private slots:
        void initSlot();

    private:
        QBuffer buffer_;
    };

}}

prefix_ pykit::detail::InternalServerReply::
InternalServerReply(QNetworkAccessManager::Operation operation,
                    QNetworkRequest const & networkRequest, Publisher * publisher)
{
    setRequest(networkRequest);
    setOperation(operation);
    setUrl(networkRequest.url());
    open(ReadWrite | Unbuffered);
    buffer_.open(ReadWrite | Unbuffered);

    Request request (*this);
    publisher->publish(request);

    seek(0);
    setHeader(QNetworkRequest::ContentLengthHeader, QVariant(size()));
    setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/html; charset=UTF-8"));

    // The following rigamole is needed to delay emitting readyRead() / finished() after until the
    // connections to those signals have been set up.
    //
    // We use a queued signal here. This will call initSlot after processing has returned to the
    // event loop.
    connect(this, SIGNAL(initSignal()), this, SLOT(initSlot()), Qt::QueuedConnection);
    emit initSignal();
}

prefix_ void pykit::detail::InternalServerReply::initSlot()
{
    emit readyRead();
    emit finished();
}

prefix_ bool pykit::detail::InternalServerReply::atEnd()
    const
{ return buffer_.atEnd(); }

prefix_ qint64 pykit::detail::InternalServerReply::bytesAvailable()
    const
{ return buffer_.bytesAvailable(); }

prefix_ qint64 pykit::detail::InternalServerReply::bytesToWrite()
    const
{ return buffer_.bytesToWrite(); }

prefix_ bool pykit::detail::InternalServerReply::canReadLine()
    const
{ return buffer_.canReadLine(); }

prefix_ bool pykit::detail::InternalServerReply::isSequential()
    const
{ return buffer_.isSequential(); }

prefix_ qint64 pykit::detail::InternalServerReply::pos()
    const
{ return buffer_.pos(); }

prefix_ bool pykit::detail::InternalServerReply::reset()
{ return buffer_.reset(); }

prefix_ bool pykit::detail::InternalServerReply::seek(qint64 pos)
{ return buffer_.seek(pos); }

prefix_ qint64 pykit::detail::InternalServerReply::size()
    const
{ return buffer_.size(); }

prefix_ bool pykit::detail::InternalServerReply::waitForBytesWritten(int msecs)
{ return buffer_.waitForBytesWritten(msecs); }

prefix_ bool pykit::detail::InternalServerReply::waitForReadyRead(int msecs)
{ return buffer_.waitForReadyRead(msecs); }

prefix_ qint64 pykit::detail::InternalServerReply::readData(char * data, qint64 maxSize)
{ return buffer_.read(data,maxSize); }

prefix_ qint64 pykit::detail::InternalServerReply::writeData(char const * data, qint64 maxSize)
{ return buffer_.write(data,maxSize); }

prefix_ void pykit::detail::InternalServerReply::abort()
{}

prefix_ void pykit::detail::InternalServerReply::clearResponse()
{
    buffer_.buffer().clear();
}

///////////////////////////////////////////////////////////////////////////
// pykit::Request

prefix_ void pykit::Request::write(std::string const & data)
{
    reply_.write(QByteArray(data.data(),data.size()));
}

prefix_ void pykit::Request::reset()
{
    reply_.clearResponse();
}

prefix_ void pykit::Request::setContentType(std::string const & contentType)
{
    reply_.setHeader(QNetworkRequest::ContentTypeHeader, QVariant(contentType.c_str()));
}

prefix_ void pykit::Request::setLocation(std::string const & location)
{
    reply_.setHeader(QNetworkRequest::LocationHeader, QVariant(location.c_str()));
}

prefix_ QUrl pykit::Request::url()
    const
{
    return reply_.url();
}

prefix_ pykit::Request::Request(detail::InternalServerReply & reply)
    : reply_ (reply)
{}

///////////////////////////////////////////////////////////////////////////
// pykit::InternalNetworkAccessManager

prefix_ pykit::InternalNetworkAccessManager::
InternalNetworkAccessManager(QNetworkAccessManager * manager, QObject * parent,
                             Publisher * publisher)
    : QNetworkAccessManager(parent), publisher_ (publisher)
{
    setCache(manager->cache());
    setCookieJar(manager->cookieJar());
    setProxy(manager->proxy());
    setProxyFactory(manager->proxyFactory());
}

prefix_ QNetworkReply *
pykit::InternalNetworkAccessManager::createRequest(Operation operation,
                                                        QNetworkRequest const & request,
                                                        QIODevice * device)
{
    if (request.url().scheme() != "srv")
        return QNetworkAccessManager::createRequest(operation, request, device);

    return new detail::InternalServerReply(operation, request, publisher_);
}

#include "Publisher.moc"

///////////////////////////////cc.e////////////////////////////////////////
#undef prefix_
//#include "Publisher.mpp"


// Local Variables:
// mode: c++
// fill-column: 100
// comment-column: 40
// c-file-style: "j32"
// indent-tabs-mode: nil
// ispell-local-dictionary: "american"
// compile-command: "scons -U"
// End:
